#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from gettext import gettext as _
import argparse

from gi import require_version
require_version("Gtk", "3.0")

from gi.repository import Gtk
from gi.repository import Gio
from gi.repository import Gdk

from gasimporter.configuration import Configuration as Config
from gasimporter.window import ApplicationWindow
from gasimporter.window import PrefernecesWindow
from gasimporter.window import ShortcutsWindow
from gasimporter.window import AboutWindow

class Application(Gtk.Application):

    def __init__(self):
        Gtk.Application.__init__(self,
                                 flags = Gio.ApplicationFlags.HANDLES_COMMAND_LINE)

        self.window = None
        self.args = None

    """
    Gtk.Application startup handler
    """
    def do_startup(self):
        Gtk.Application.do_startup(self)

        self.build_menu()

    """
    Gtk.Application activate handler
    The starts the window if one does not exist and shows them all.
    """
    def do_activate(self):
        # Window already exists, return that
        if self.window:
            return

        # Else create a new application window.
        self.window = ApplicationWindow(self)

        # Connect quit event (normally already done but lets do it anyway)
        self.window.connect("delete_event", self.on_quit)

        # Build UI
        self.window.build()

        # Show all the windows.
        # .show() vs. .show_all() vs. .present()
        # You MUST use .show_all() for Gtk.headerBar() to work!
        # .present() is a wrapper of .show() and .show() onyl shows the current window.
        self.window.show_all()

    """
    Gtk.Application command line handler
    Called if Gio.ApplicationFlags.HANDLES_COMMAND_LINE is set.
    """
    def do_command_line(self, args):
        Gtk.Application.do_command_line(self, args)

        # Start parsing
        parser = argparse.ArgumentParser(description=Config.APP_DESCRIPTION,
                                         prog=Config.APP_COMMAND)

        # Add debug option (-d/--debug)
        parser.add_argument('-d', '--debug',
                            action='store_true',
                            help=_("Run the program in debug mode and show extra output"))

        # Store arguments so application can use them
        # We strip off the first argument (the file name)
        self.args = parser.parse_args(args.get_arguments()[1:])

        # Since we are calling do_command_line() we need to call do_activate()
        # For the program to continue
        self.do_activate()

        return 0

    """
    Build Gio Menu and add acclerator actions.
    """
    def build_menu(self):
        # Init Gio menu (app-menu)
        menu = Gio.Menu()

        # Menu info
        actions = [
            ("preferences", _("Preferences"), self.on_preferences,["<Control>p", "<Control>comma"], _("Open preferences window.")),
            ("shortcuts", _("Keyboard Shortcuts"), self.on_shortcuts, ["F2"], _("Open shortcuts window.")),
            ("help", _("Help"), self.on_help, ["F3"], _("Open help window.")),
            ("about", _("About"), self.on_about, ["F1"], _("Open about window.")),
            ("quit", _("Quit"), self.on_quit, ["<Control>q"], _("Quit application.")),
        ]

        # Build callbacks and acclerators
        for action, title, callback, accelerators, description in actions:
            # Create menu item
            menu.append(title, "app.{}".format(action))
            # Add action to menu item
            simpleAction = Gio.SimpleAction.new(action, None)
            simpleAction.connect('activate', callback)
            self.add_action(simpleAction)
            # Add accelerator action
            if accelerators:
                self.set_accels_for_action("app.{}".format(action), accelerators)
            # Add acclerator to shortcut window
            if description:
                ShortcutsWindow.add_action("app.{}".format(action), description)

        # Add app-menu to app
        self.set_app_menu(menu)

    """
    Callbacks for Gio.Menu
    """
    def on_preferences(self, action = None, param = None):
        preferences = PrefernecesWindow(main_window=self.window)
        preferences.build()
        preferences.show_all()

    def on_help(self, action = None, param = None):
        Gtk.show_uri(None, Config.APP_MANUAL_URL_OFFLINE, Gdk.CURRENT_TIME)

    def on_shortcuts(self, action = None, param = None):
        shortcuts = ShortcutsWindow(main_window=self.window, app=self)
        shortcuts.build()
        shortcuts.show_all()

    def on_about(self, action = None, param = None):
        about = AboutWindow(main_window=self.window)
        about.build()
        about.show_all()

    """
    Gtk.Application shutdown handler
    Do clean up before the application is closed, useful when you need
    to close a database connection or save work for the application.
    This is triggered when self.quit() is called.
    """
    def do_shutdown(self):
        Gtk.Application.do_shutdown(self)

    """
    Gtk.Application quit handler
    Quit the application when requested.
    """
    def on_quit(self, action = None, param = None):
        self.quit()
